<?php

namespace Drupal\preview_link;

use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Calculates link expiry time.
 */
class LinkExpiry {

  /**
   * Default expiry time in seconds. This amounts to 7 days.
   *
   * @var int
   */
  const DEFAULT_EXPIRY_SECONDS = 604800;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Statically cache the lifetime.
   *
   * @var int
   */
  protected $lifetime;

  /**
   * LinkExpiry constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * Calculates default lifetime of a preview link.
   *
   * @return int
   *   Preview link in seconds.
   */
  public function getLifetime() {
    if (!isset($this->lifetime)) {
      $config = $this->configFactory->get('preview_link.settings');
      $this->lifetime = $config->get('expiry_seconds') ?: self::DEFAULT_EXPIRY_SECONDS;
    }

    return $this->lifetime;
  }

}
